package luckdraw

import (
	"errors"
	"net/http"
	"spaceofcheng/services/controllers"
	"spaceofcheng/services/models/customer"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/service/luckdraw"
	"spaceofcheng/services/utils"
)

// LuckdrawController 应用
type LuckDrawController struct {
	serv    *luckdraw.LuckdrawServ
	custDAO *customer.CustomerDAO
	controllers.BaseController
}

// Constructor 初始化 Controller
// @Title Constructor
// @Description 初始化 Controller, 系统自动调用
func (c *LuckDrawController) Constructor() {
	c.serv = luckdraw.NewLuckdrawServ(c.Context)
	c.custDAO = customer.NewCustomerDAO(c.Context)
}

// GetUserByCode 根据code获取用户信息
func (c *LuckDrawController) GetUserByCode() {
	userRaw := c.Context.Get("customer")
	if userRaw == nil {
		c.ResponseError(errors.New("系统内部错误"), http.StatusInternalServerError)
	}

	user := userRaw.(model.TaCustomer)
	luckdrawid := c.GetString(":id")
	record, err := c.serv.GetUserLuckDrawByLuckDraw(user.CustomerId, luckdrawid)
	if err != nil {
		utils.LogError("获取用户抽奖信息失败: " + err.Error())
		c.ResponseError(errors.New("获取用户抽奖信息失败"))
	}
	var detail *model.TaPrizeDetail
	if record != nil {
		detail, err = c.serv.GetDetailByRecord(record.Id)
		if err != nil {
			utils.LogError("获取用户抽奖信息失败: " + err.Error())
			c.ResponseError(errors.New("获取用户抽奖信息失败"))
		}
	}

	c.ResponseJSON(map[string]interface{}{
		"user":   user,
		"record": record,
		"detail": detail,
	})
}

// GetLuckDraw 获取抽奖信息
func (c *LuckDrawController) GetLuckDraw() {
	id := c.GetString(":id")
	luckdraw, err := c.serv.GetLuckDraw(id)
	if err != nil {
		utils.LogError("获取抽奖信息失败: " + err.Error())
		c.ResponseError(errors.New("获取抽奖信息失败"))
	}
	c.ResponseJSON(luckdraw)
}

// LuckDraw 抽奖
func (c *LuckDrawController) LuckDraw() {
	id := c.GetString(":id")
	userRaw := c.Context.Get("customer")
	if userRaw == nil {
		c.ResponseError(errors.New("系统内部错误"), http.StatusInternalServerError)
	}

	user := userRaw.(model.TaCustomer)

	prize, detail, err := c.serv.LuckDraw(id, &user)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(map[string]interface{}{
		"prize":  prize,
		"detail": detail,
	})
}

// GetRecordByLuckDraw 获取抽奖记录
func (c *LuckDrawController) GetRecordByLuckDraw() {
	id := c.GetString("id")
	records, err := c.serv.GetRecordByLuckDraw(id)
	if err != nil {
		utils.LogError("获取抽奖记录失败: " + err.Error())
		c.ResponseError(errors.New("获取抽奖记录失败"))
	}
	c.ResponseJSON(records)
}

// GetUserLuckDraw 获取用户的抽奖记录
func (c *LuckDrawController) GetUserLuckDraw() {
	userid := c.GetString("userid")
	records, err := c.serv.GetUserLuckDraw(userid)
	if err != nil {
		utils.LogError("获取用户抽奖记录失败: " + err.Error())
		c.ResponseError(errors.New("获取用户抽奖记录失败"))
	}
	c.ResponseJSON(records)
}

// GetRecordByID 根据ID获取中奖记录信息
func (c *LuckDrawController) GetRecordByID() {
	id := c.GetString(":id")
	record, err := c.serv.GetRecordByID(id)
	if err != nil {
		utils.LogError("获取中奖记录失败: " + err.Error())
		c.ResponseError(errors.New("获取中奖记录失败"))
	}

	caseAddress, err := c.serv.GetCaseAddressByLuckDarw(record.LuckdrawId)
	if err != nil {
		utils.LogError("获取案场信息失败: " + err.Error())
		c.ResponseError(errors.New("获取案场信息失败"))
	}

	if caseAddress == "" {
		c.ResponseError(errors.New("无案场, 或案场地址没有维护"))
	}

	prize, err := c.serv.GetPrizeByID(record.PrizeId)
	if err != nil {
		utils.LogError("校验奖品记录失败: " + err.Error())
		c.ResponseError(errors.New("校验奖品记录失败"))
	}

	c.ResponseJSON(map[string]interface{}{
		"record":      record,
		"prize":       prize,
		"caseAddress": caseAddress,
	})
}

func (c *LuckDrawController) SaveShareRecord() {
	errIgnore := "error-ignore"

	id := c.GetString(":id")

	fromID := c.GetString("from")
	if fromID == "" {
		c.ResponseData(
			errIgnore,
			"没有分享人ID",
			http.StatusBadRequest,
		)
	}

	from, err := c.custDAO.GetCustomerByID(fromID)
	if err != nil {
		utils.LogError("获取分享人信息失败: " + err.Error())
		c.ResponseData(
			errIgnore,
			"获取分享人信息失败",
			http.StatusBadRequest,
		)
	}

	if from == nil || from.CustomerId == "" {
		utils.LogError("分享人信息不存在: " + err.Error())
		c.ResponseData(
			errIgnore,
			"分享人信息不存在",
			http.StatusBadRequest,
		)
	}

	current := c.Context.Get("customer").(model.TaCustomer)

	// 自己分享给自己
	if from.CustomerId == current.CustomerId {
		c.ResponseData(
			errIgnore,
			"本人分享内容无效",
			http.StatusBadRequest,
		)
	}

	luckdraw, err := c.serv.GetLuckDrawByID(id)
	if err != nil {
		utils.LogError("获取抽奖信息失败: " + err.Error())
		c.ResponseData(
			errIgnore,
			"获取抽奖信息失败",
			http.StatusBadRequest,
		)
	}

	if luckdraw == nil {
		utils.LogError("获取抽奖信息失败: " + err.Error())
		c.ResponseData(
			errIgnore,
			"没有找到 id ("+id+") 对应的抽奖信息",
			http.StatusBadRequest,
		)
	}

	existsRec, err := c.serv.GetLuckDrawShareData(from.CustomerId, current.CustomerId, luckdraw.Id, luckdraw.CaseId)
	if err != nil {
		utils.LogError("获取已有分享信息失败: " + err.Error())
		c.ResponseData(
			errIgnore,
			"获取已有分享信息失败",
			http.StatusBadRequest,
		)
	}

	if existsRec != nil && len(existsRec) > 0 {
		c.ResponseData(
			errIgnore,
			"已分享记录, 再次分享无效",
			http.StatusBadRequest,
		)
	}

	shareDt := model.TaShareLuckyRecord{
		FromCustomerId:     from.CustomerId,
		FromCustomerName:   from.CustomerName,
		FromCustomerWxname: from.Name,
		FromCustomerTel:    from.Phone,
		ToCustomerId:       current.CustomerId,
		ToCustomerName:     current.CustomerName,
		ToCustomerWxname:   current.Name,
		ToCustomerTel:      current.Phone,
		CaseId:             luckdraw.CaseId,
		LuckydrawId:        luckdraw.Id,
		LuckydrawName:      luckdraw.Name,
	}

	if err := c.serv.SaveLuckDrawShareData(&shareDt); err != nil {
		utils.LogError("保存分享记录失败: " + err.Error())
		c.ResponseData(
			errIgnore,
			"保存分享记录失败",
			http.StatusBadRequest,
		)
	}

	c.ResponseJSON("ok")
}
